from pydantic import BaseModel, Field
from typing_extensions import Literal, Optional

from Schemas_ses.enumeration.enums import StudiesLevel, LocalType
from Schemas_ses.models.model import AnnexeModel
from Schemas_ses.type.types import Year


class TeacherFurniture(BaseModel):
    teacher_desk_with_chair: Optional[int]
    cupboard_or_closet: Optional[int]


Good_or_bad = Literal['Bon / Acceptable', 'Mauvais']


class Office(AnnexeModel):
    name: str
    study_years: Optional[StudiesLevel]
    local_type: Optional[LocalType]
    year_of_commissioning: Optional[Year]
    is_unused: Optional[bool]
    wall_material: Optional[Literal["En dur", "Semi-dur / banco", "Autre: Planche / bambou", "Sans mur"]]
    wall_condition: Optional[Good_or_bad]
    roof_material: Optional[Literal["Tôles", "Tuiles / Fibro ciment / Dalles", "Banco", "Paille", "Sans toit"]]
    roof_condition: Optional[Good_or_bad]
    door_material: Optional[Literal["Métallique", "Tôle / Bois", "Non installées"]]
    wall_nature: Optional[Literal["Persiennes", "Volets", "Claustras", "Non installées"]]
    funding: Optional[Literal["Collectivités locales", "APE", "Aide extérieure", "Autres / Non déterminé"]]
    teacher_furniture: TeacherFurniture
    # add student_furniture
    surface_area: Optional[float]
    blackboard: Optional[int]


class OfficeAndFurniture(AnnexeModel):
    """Se réfère à la section Locaux Et Mobiliers"""
    offices: Optional[list[Office]]



from fastapi import HTTPException
from pydantic_core import ValidationError
from pymongo.database import Database
from pymongo.results import UpdateResult
from starlette import status
from typing import get_type_hints

from Schemas_ses.models.model import Model
from Schemas_ses.models.objectid import PydanticObjectId
from Schemas_ses.models.schools.ExamResullt import ExamResult
from Schemas_ses.models.schools.FinancialData import FinancialData
from Schemas_ses.models.schools.GeneralInfo import GeneralInfo
from Schemas_ses.models.schools.IdAndLocation import IdAndLocation
from Schemas_ses.models.schools.OfficeAndFurniture import OfficeAndFurniture
from Schemas_ses.models.schools.ParticipatorySchoolManagement import ParticipatorySchoolManagement
from Schemas_ses.models.schools.SocioEnvironnement import SocioEnvironnement
from Schemas_ses.models.schools.StudentsRecord import StudentsRecord
from Schemas_ses.models.schools.TeachingAndLearningEquipment import TeachingAndLearningEquipment
from Schemas_ses.models.schools.TeachingStaff import TeachingStaff


class Schools(Model):
    name: str
    school_id: str
    id_and_location: IdAndLocation
    socio_environnement: SocioEnvironnement
    general_info: GeneralInfo
    office_and_furniture: OfficeAndFurniture
    teaching_and_learning_equipment: TeachingAndLearningEquipment
    teaching_staff: TeachingStaff
    exam_results: ExamResult
    participatory_school_management: ParticipatorySchoolManagement
    financial_data: FinancialData
    students_record: StudentsRecord

    @classmethod
    def find_one(cls, database: Database, mask: dict):
        if result := database.Schools.find_one(mask):
            # Exclude _id from the result

            return Schools(**result)
        else:
            return None

    @classmethod
    def find_one_or_404(cls, database: Database, mask: dict):
        if result := database.Schools.find_one(mask):
            # Exclude _id from the result

            return Schools(**result)
        else:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail="Utilisateur non trouvé",
            )

    def save(self):
        data = self.to_bson()
        result = self.database.Schools.insert_one(data)
        self.id = PydanticObjectId(result.inserted_id)

    def full_update(self, data: dict):
        self.database.Schools.update_one({"_id": self.id},
                                         {"$set": data})
        self.__init__(**Schools.find_one_or_404(self.database, {"_id": self.id}).dict())

    def partial_update(self, data: dict, section: str) -> UpdateResult:
        try:
            section_model = get_type_hints(Schools)
            validator = section_model[section]
            validator(**data)
            result = self.database.Schools.update_one({"_id": self.id},
                                                      {"$set": {section: data}})
            self.__init__(**Schools.find_one_or_404(self.database, {"_id": self.id}).dict())
            return result
        except ValidationError as e:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail=e.errors()
            )





from typing_extensions import Literal, Optional
from datetime import date

from Schemas_ses.enumeration.enums import StudiesLevel
from Schemas_ses.models.model import AnnexeModel
from Schemas_ses.type.types import Year, Sexe


class HighestDegree(AnnexeModel):
    academic: Optional[Literal["CEP", "BEPC", "BAC", "LICENCE", "MASTER"]]
    professional: Optional[Literal["CEAP", "CAP"]]


class Seniority(AnnexeModel):
    public: Optional[int]
    department: Optional[int]
    school: Optional[int]


class NumberOfVisit(AnnexeModel):
    inspector: Optional[int]
    director: Optional[int]
    cp: Optional[int]


class Personnel(AnnexeModel):
    matricule: str
    name: str
    surname: str
    sexe: Sexe
    birth_year: Optional[Year]
    highest_degree: HighestDegree
    grade: Optional[str]
    statut: Optional[Literal["ACPDE", "ACE", "AME", "Communautaire", "Fonctionnaire de l'état", "Privé", "Autre"]]
    fonction: Optional[Literal["Directeur", "Adjoint", "Enseignant"]]
    real_indice: Optional[int]
    teached_subjects: Optional[list[StudiesLevel]]
    seniority: Seniority
    last_formation_date: Optional[date]
    number_of_visit: NumberOfVisit
    up_participation: Optional[int]


class TeachingStaff(AnnexeModel):
    personnel: Optional[list[Personnel]]
